;
$DEBUG
$DATE (30.08.2004)
$TITLE (Monitor fuer Atmel 89S8252)
$XREF
$PAGELENGTH(56)
$PAGEWIDTH(150)

$NOLIST
$NOMOD51
$INCLUDE(89S8252.MCU)
$LIST

;  Mikrocontroller-Monitorprogramm
;  Version fr Atmel 89S8252
;  ==========================
;
;  Basis: Elektor-8051-Monitor (Vers. EMON52) (Martin Ohsmann)
;  Version fr Experimentier-Board (Eagle-Dok. AT89S8252-D/S)

;  geladene Programme mssen bei der Speicheradresse $8000 beginnen
;  nderungen:  - Startadresse bei $8000
;               - Interrupts bei $8003 ff bleiben nach Reset erhalten,
;               - Code-Umsetzung auf ASEM51
;               - neuer Befehl : G = automatischer Start bei $8000 
;               - Anzeige der Register ("S")
;               - Stackpointer Initialisierung ergnzt
;               - prft beim Start, ob RxD (P3.0) = 0 ist.
;                 St1/3 muss ber 10k mit VCC verbunden sein
;                 ==> PC angeschlossen: P3.0 = 1 ==> starte Monitor
;                     kein PC:          P3.0 = 0 ==> starte Anwenderprog. bei 8000H

; Gre: < 4 kB

;    J. Rathlev, IEAP-Kiel, Aug. 2004
;
;------------------------------------------------------------------
;  Serielle Schnittstelle:
; Baudrateneinstellungen:
; SMOD = 1 ==> TH1 = 256 - (2 * f/Hz) / (384 * BR)

; 22,1184 MHz -  2400 : 208     24 MHz - 2400 : 204        12 MHz - 2400 : 230
;             -  4800 : 232            - 4800 : 230               - 4800 : 243
;             -  9600 : 244            - 9600 : 243
;             - 19200 : 250
;             - 38400 : 253

BaudCount       EQU     0               ; automatische Erkennung (siehe AutoBaud)
;BaudCount       EQU     244               ; 9600 (s.o.)

;------------------------------------------------------------------
; Delay-Routine TIME0:
; Total = 5 * (n1+1) cyles  
;   12 MHz:      NL =  96 ==> 485 s
;   22,1184 MHz: NL = 178 ==> 485 s
;   24 MHz:      NL = 193 ==> 485 s

        NL      EQU   178


;--------------------------------------------------------------------
; Speicheradresse fr Code-RAM

CodeRAM  EQU  8000H

         DSEG
;
; RAM Definitions
;
       ORG 030H

COMMAND:        DS   1          ; for MON call , MUST be address 030H !!!!!!
CNT1:           DS   1
HEXLEN:         DS   1

INTELl          EQU HEXLEN      ; intel record length [52]
HEXSUM:         DS   2
INTELs          EQU HEXSUM      ; intel checksum  [52]
INTELt          EQU HEXSUM+1    ; intel record type [52]


dBYTS           EQU  5           ; must be 5 , loop unrolling used ;
dREG1:          DS   dBYTS
dACCU:          DS   dBYTS

;                       ; used for snapshot and MON
PSWSAV:         DS   1
ACCSAV:         DS   1
DPHSAV:         DS   1
DPLSAV:         DS   1
BaudSave:        DS      4             ;save baud for warm boot, 4 bytes

;
;------------------------------------------------------------------
;
; arithmetic storage
;
top     EQU   dREG1      ; overlay 2 bytes
next    EQU   dREG1+2    ; overlay 2 bytes
;
FAC1    EQU top          ; overlay factors fo 16*16 Bit
FAC2    EQU next         ; overlay
DIVI    EQU FAC2         ; overlay divisor for 32/16 bit division
QUOT    EQU FAC1         ; overlay quotient for 32/16 bit division
PROD    EQU dACCU        ; overlay 5 bytes ( 4+ extension) for division
DIVCNT: DS 1
;
Rtop    EQU  $
Stack   EQU 0FFH-18H
;
;--------------------------------------------------------------------
;
        CSEG

        ORG 0000
        LJMP  START
 ;
 ;      all interrupts are linked via LJMP to CodeRAM
 ;
        ORG 0003H        ; use jump table at address CodeRAM
        LJMP CodeRAM+03H
 ;
        ORG 000BH
        LJMP CodeRAM+0BH
 ;
        ORG 0013H
        LJMP CodeRAM+13H
 ;
        ORG 001BH
        LJMP CodeRAM+1BH
 ;
        ORG 0023H
        LJMP CodeRAM+23H
 ;
        ORG 002BH
        LJMP CodeRAM+2BH

 ; only for 592
        ORG 0033H
        LJMP CodeRAM+33H
 ;
        ORG 003BH
        LJMP CodeRAM+3BH

 ; for 535 / 537 / 592
        ORG 0043H
        LJMP CodeRAM+43H
 ;
        ORG 004BH
        LJMP CodeRAM+4BH
 ;
        ORG 0053H
        LJMP CodeRAM+53H
 ;
        ORG 005BH
        LJMP CodeRAM+5BH
 ;
        ORG 0063H
        LJMP CodeRAM+63H
 ;
        ORG 006BH
        LJMP CodeRAM+6BH
 ;
        ORG 0073H
        LJMP CodeRAM+73H
 ;
        ORG 0083H
        LJMP CodeRAM+83H
 ;
        ORG 009BH
        LJMP CodeRAM+9BH
 ;
 ;      compatibility to MON1
 ;
        ORG   100H
        LJMP  SND      ; call to subroutine "Send character"
        LJMP  LINK     ;                    "Link interrupts to default"
        LJMP  STXT     ;                    "Send text"
 ;
 ;      entry point for MON-CALLS
 ;
        ORG   0200H
        LJMP  MON
;
;      main program starts here
;
START:  MOV   SP,#Stack     ; set Stackpointer
        MOV   PSW,#0          ; reset register banks
        LCALL AutoBaud       ; detect baudrate
        MOV   PCON,#080H      ; SMOD=1
        MOV   TMOD,#22H       ; modes are timer

        MOV   A,BaudSave+3    ; BR-Count (s.o.)
        MOV   TH1,A           ; preload value
        MOV   TL1,A
        SETB  TCON.6          ; start counter
        MOV   SCON,#052H      ; mode 1 , Enable receiver=10H
;
        JB    P3.0,StMon      ; test if RxD is connected to 5 Volts ?
                              ; if so go to run mode else start monitor

; Run mode
        MOV   DPTR,#UPTXT
        LCALL STXT
        LJMP  CodeRAM


;  Set Interrupt links in CodeRAM to RMON

;                  C:        8051 / 535 / 537 / 8252                  
StMon:  MOV   A,#0
        MOV   DPTR,#START
        LCALL LINK
        MOV   A,#3
        MOV   DPTR,#INTE0   ; External 0 / ## / ## /##
        LCALL LINK
        MOV   A,#0BH
        MOV   DPTR,#INTF0   ; Timer 0 overflow / ## / ## /##
        LCALL LINK
        MOV   A,#13H
        MOV   DPTR,#INTE1   ; External 1 / ## / ## /##
        LCALL LINK
        MOV   A,#1BH
        MOV   DPTR,#INTF1   ; Timer 1 overflow / ## / ## /##
        LCALL LINK
        MOV   A,#23H
        MOV   DPTR,#INTRT   ; Serial port / ## / ## /##
        LCALL LINK
        MOV   A,#2BH
        MOV   DPTR,#INTCAN  ; -- / Timer 2 overflow / ## / ##   
        LCALL LINK
        MOV   A,#33H
        MOV   DPTR,#INTCT0  ; -- / -- / -- / --
        LCALL LINK
        MOV   A,#3BH
        MOV   DPTR,#INTCT1  ; -- / -- / -- / --
        LCALL LINK
        MOV   A,#43H
        MOV   DPTR,#INTADC  ; -- / A/D converter / --
        LCALL LINK
        MOV   A,#4BH
        MOV   DPTR,#INTEX2  ; -- / External 2 /  -- / --
        LCALL LINK
        MOV   A,#53H
        MOV   DPTR,#INTEX3  ; -- / External 3 /  -- / --
        LCALL LINK
        MOV   A,#5BH
        MOV   DPTR,#INTEX4  ; -- / External 4 /  -- / --
        LCALL LINK
        MOV   A,#63H
        MOV   DPTR,#INTEX5  ; -- / External 5 /  -- / --
        LCALL LINK
        MOV   A,#6BH
        MOV   DPTR,#INTEX6  ; -- / External 6 /  -- / --
        LCALL LINK
        MOV   A,#73H
        MOV   DPTR,#INTF2   ; -- / -- / -- / --
        LCALL LINK
        MOV   A,#83
        MOV   DPTR,#INTRT1  ; -- / -- / Serial Port 2 / --
        LCALL LINK
        MOV   A,#9BH
        MOV   DPTR,#INTCTF  ; -- / -- / Compare timer overflow / --
        LCALL LINK

        LJMP  DIALOG          ; start interactive mode

UPTXT:  DB    '  LJMP TO CodeRAM...',13,10,0
TXT0:   DB    13,10,13,10
        DB    '  --RMON 8252-- Version 1.0 (Aug. 2004) RAMTOP=',0

HlpTxt: DB    13,10,'Commands:',13,10
        DB    '  P[{adr}]: Show code memory',13,10
        DB    '  D[{adr}]: Show external data memory',13,10
        DB    '  R[{adr}]: Show internal data memory',13,10
        DB    '  X[{adr}]: Start at code address',13,10
        DB    '  F{adr} [{val} ..]: Change byte(s) in ext. data memory',13,10
        DB    '  M{adr} [{val} ..]: Change byte(s) in int. data memory',13,10
        DB    '     + {Enter} ({adr], {val} are hex values, [..] is optional)', 13,10
        DB    '  H : show last memory address',13,10
        DB    '  G : Start at 8000H',13,10,0

ITXT03: DB    '  INTERRUPT, IE0',13,10,0
ITXT0B: DB    '  INTERRUPT, TF0',13,10,0
ITXT13: DB    '  INTERRUPT, IE1',13,10,0
ITXT1B: DB    '  INTERRUPT, TF1',13,10,0
ITXT23: DB    '  INTERRUPT, RI/TI',13,10,0
ITXT2B: DB    '  INTERRUPT, TF2/CAN',13,10,0
ITXT33: DB    '  INTERRUPT, -/CT0',13,10,0
ITXT3B: DB    '  INTERRUPT, -/CT1',13,10,0
ITXT43: DB    '  INTERRUPT, ADC/CT2',13,10,0
ITXT4B: DB    '  INTERRUPT, EX2/CT3',13,10,0
ITXT53: DB    '  INTERRUPT, EX3/ADC',13,10,0
ITXT5B: DB    '  INTERRUPT, EX4/CM0',13,10,0
ITXT63: DB    '  INTERRUPT, EX5/CM1',13,10,0
ITXT6B: DB    '  INTERRUPT, EX6/CM2',13,10,0
ITXT73: DB    '  INTERRUPT, -/TF2',13,10,0
ITXT83: DB    '  INTERRUPT, RI1/TI1',13,10,0
ITXT9B: DB    '  INTERRUPT, CTF',13,10,0
;
;      default interrupt handlers
;
INTE0:  MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT03
        LCALL STXT
        LJMP  STOP1

INTE1:  MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT13
        LCALL STXT
        LJMP  STOP1

INTF0:  MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT0B
        LCALL STXT
        LJMP  STOP1

INTF1:  MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT1B
        LCALL STXT
        LJMP  STOP1

INTRT:  MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT23
        LCALL STXT
        LJMP  STOP1

INTCAN: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT2B
        LCALL STXT
        LJMP  STOP1

INTCT0: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT33
        LCALL STXT
        LJMP  STOP1

INTCT1: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT3B
        LCALL STXT
        LJMP  STOP1

INTADC: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT43
        LCALL STXT
        LJMP  STOP1

INTEX2: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT4B
        LCALL STXT
        LJMP  STOP1

INTEX3: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT53
        LCALL STXT
        LJMP  STOP1

INTEX4: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT5B
        LCALL STXT
        LJMP  STOP1

INTEX5: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT63
        LCALL STXT
        LJMP  STOP1

INTEX6: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT6B
        LCALL STXT
        LJMP  STOP1

INTF2:  MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT73
        LCALL STXT
        LJMP  STOP1

INTRT1: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT83
        LCALL STXT
        LJMP  STOP1

INTCTF: MOV   IE,#0           ; disable all interrups
        MOV   DPTR,#ITXT9B
        LCALL STXT
        LJMP  STOP1
;
STOP1:  LJMP START
STOP:   SJMP STOP
;
;--------------------------------------------------------------------
;
; interactive command interpreter
;
DIALOG: MOV   DPTR,#TXT0
        LCALL STXT            ; start prompt
        MOV   A,#Rtop
        LCALL BYTE
CMND:   LCALL CRLF            ; main interpreter loop
        MOV   A,#'#'          ; prompt
        LCALL SND
CMND2:  LCALL GETCHR          ; get command character
        CJNE  A,#':',case2    ; and compare 
        LCALL INTEL           ; load Hex-Intel
        SJMP  CMND2
case2:  LCALL UPCASE
        CJNE  A,#'X',case7    
        LCALL EXEC            ; start program
        SJMP  CMND
case7:  CJNE  A,#'G',case1    
        LCALL EXCodeRAM       ; start at CodeRAM
        SJMP  CMND
case1:  CJNE  A,#'H',case0
        LJMP  SADR            ; show last address
case0:  CJNE  A,#'D',case3
        LJMP  DDSP            ; show data
case3:  CJNE  A,#'P',case4
        LJMP  PDSP            ; show programm
case4:  CJNE  A,#'R',case5
        LJMP  RDSP            ; show ram
case5:  CJNE  A,#'F',case6
        LJMP  FILL1           ; fill data
case6:  CJNE  A,#'M',case8
        LJMP  xFILL1          ; modify ram
case8:  CJNE  A,#'S',case9
        MOV   ACCSAV,A        ; do not change these lines !
        MOV   PSWSAV,PSW
        LCALL SNAP            ; show registers
        LJMP  CMND
case9:  CJNE  A,#'?',CMND
        LJMP  HELP
;
;--------------------------------------------------------------------
;
; Download INTEL
;
;
INTEL:  MOV   INTELs,#0
        LCALL GET8       ; one 8-bit hex value
        JC    INTELe
        MOV   INTELl,A   ; length
        LCALL CHK        ; comp. checksum
        LCALL GET8
        JC    INTELe
        MOV   DP0H,A      ; address MSB
        LCALL CHK
        LCALL GET8
        JC    INTELe
        MOV   DP0L,A      ; address LSB
        LCALL CHK
        LCALL GET8
        JC    INTELe
        MOV   INTELt,A   ; record type
        LCALL CHK
        MOV   A,INTELt
        CJNE  A,#01,noendI ; end block ?
iCHK:   LCALL GET8      ; yes - check the checksum
        JC    INTELe    ; error
        LCALL CHK
        MOV   A,INTELs
        JNZ   INTELe2   ; checksum error
        RET

; read data
noendI: CJNE  A,#00,INTELe
Iloop:  MOV   A,INTELl ; remaining Bytes
        JZ    iCHK
        DEC   INTELl
        LCALL GET8     ; data
        JC    INTELe
        MOVX  @DPTR,A  ; stor
        INC   DPTR
        LCALL CHK
        SJMP  Iloop

; error
INTELe: MOV DPTR,#INTEL0
        LCALL STXT
        RET
;
INTELe2: MOV DPTR,#INTEL1
        LCALL STXT
        RET
;
INTEL0: DB 13,10,'INTEL LOAD format ERROR !',0
INTEL1: DB 13,10,'INTEL LOAD checksum ERROR !',0
;
CHK:    ADD    A,INTELs
        MOV    INTELs,A
        RET
;
;--------------------------------------------------------------------
;
; fill data storage
;
FILL1:  LCALL GET16           ; get start address
        MOV   A,R6
        JNZ   FILL2
        LJMP  CMND            ; return if no hex number
FILL2:  MOV   DP0L,@R0
        INC   R0
        MOV   DP0H,@R0
FILL3:  MOV   A,DP0H           ; display current address
        LCALL BYTE
        MOV   A,DP0L
        LCALL BYTE
        MOV   A,#':'
        LCALL SND
        MOVX  A,@DPTR         ; display current contents
        LCALL BYTE
        LCALL BLANK
        LCALL GET16           ; get new contents
        MOV   R5,A            ; save last character
        MOV   A,R6
        JNZ   FILL4           ; change if input
        CJNE  R5,#' ',FILL5
        SJMP  FILL6           ; on BLANK go on
FILL5:  LJMP  CMND            ; exit from loop on other character
FILL4:  MOV   A,@R0           ; fetch byte and store at DPTR
        MOVX  @DPTR,A
FILL6:  LCALL CRLF
        INC   DPTR
        SJMP  FILL3
;
;--------------------------------------------------------------------
;
; fill internal ram storage
;
xFILL1: LCALL GET16           ; get start address
        MOV   A,R6
        JNZ   xFILL2
        LJMP  CMND            ; return if no hex number
xFILL2: MOV   DP0L,@R0
        INC   R0
        MOV   DP0H,@R0
xFILL3: MOV   A,DP0L           ; display current address
        LCALL BYTE
        MOV   A,#':'
        LCALL SND
        MOV   R1,DP0L
        MOV   A,@R1           ; display current contents
        LCALL BYTE
        LCALL BLANK
        LCALL GET16           ; get new contents
        MOV   R5,A            ; save last character
        MOV   A,R6
        JNZ   xFILL4           ; change if input
        CJNE  R5,#' ',xFILL5
        SJMP  xFILL6           ; on BLANK go on
xFILL5: LJMP  CMND            ; exit from loop on other character

xFILL4: MOV   R1,DP0L
        MOV   A,@R0           ; fetch byte and store at DPTR
        MOV   @R1,A
xFILL6: LCALL CRLF
        INC   DPTR
        SJMP  xFILL3
;
;--------------------------------------------------------------------
;
; show last data adress (DPTR)
;
SADR:   MOV    A,DP0H
        LCALL  BYTE
        MOV    A,DP0L
        LCALL  BYTE
        LJMP  CMND

;--------------------------------------------------------------------
;
; display data storage
;
DDSP:   LCALL GET16
        MOV   A,R6
        JZ    DDSP1           ; do not update DPTR if no input
        MOV   DP0L,@R0
        INC   R0
        MOV   DP0H,@R0
DDSP1:  MOV   R3,#8
DDSP2:  MOV   R2,#16
        LCALL DDMP1
        DJNZ  R3,DDSP2
        LJMP  CMND
;
DDMP1:  LCALL  CRLF
        MOV    A,DP0H
        LCALL  BYTE
        MOV    A,DP0L
        LCALL  BYTE
        MOV    A,#':'
        LCALL  SND
        LCALL  BLANK
DDMPL:  MOVX   A,@DPTR
        LCALL  BYTE
        LCALL  BLANK
        INC    DPTR
        DJNZ   R2,DDMPL
        RET
;
;--------------------------------------------------------------------
;
; display program storage
;
PDSP:   LCALL GET16
        MOV   A,R6
        JZ    PDSP1           ; do not update DPTR if no input
        MOV   DP0L,@R0
        INC   R0
        MOV   DP0H,@R0
PDSP1:  MOV   R3,#8
PDSP2:  MOV   R2,#16
        LCALL PDMP1
        DJNZ  R3,PDSP2
        LJMP  CMND
;
PDMP1:  LCALL  CRLF
        MOV    A,DP0H
        LCALL  BYTE
        MOV    A,DP0L
        LCALL  BYTE
        LCALL  BLANK
        MOV    A,#':'
        LCALL  SND
PDMPL:  MOV    A,#0
        MOVC   A,@A+DPTR
        LCALL  BYTE
        LCALL  BLANK
        INC    DPTR
        DJNZ   R2,PDMPL
        RET
;
;--------------------------------------------------------------------
;
; display internal RAM
;
RDSP:   LCALL GET16
        MOV   A,R6
        JZ    RDSP1           ; do not update DPTR if no input
        MOV   DP0L,@R0
        INC   R0
        MOV   DP0H,@R0
RDSP1:  MOV   R3,#8
RDSP2:  MOV   R2,#16
        LCALL RDMP1
        DJNZ  R3,RDSP2
        LJMP  CMND
;
RDMP1:  LCALL  CRLF
        MOV    A,DP0L
        LCALL  BYTE
        LCALL  BLANK
        MOV    A,#':'
        LCALL  SND
RDMPL:  MOV    R0,DP0L
        MOV    A,@R0
        LCALL  BYTE
        LCALL  BLANK
        INC    DPTR
        DJNZ   R2,RDMPL
        RET
;
;--------------------------------------------------------------------
; execute programm, ask for Address
EXEC:   LCALL GET16         ;get start address
        MOV   DP0L,@R0
        INC   R0
        MOV   DP0H,@R0
EXEC1:  MOV   PSW,#0
        MOV   R0,#0
        MOV   R1,#0
        MOV   R2,#0
        MOV   R3,#0
        MOV   R4,#0
        MOV   R5,#0
        MOV   R6,#0
        MOV   R7,#0
        MOV   B,#0
        MOV   A,#0
        JMP   @A+DPTR

;execute programm at CodeRAM+00H
EXCodeRAM: 
        MOV   DPTR,#CodeRAM
        SJMP  EXEC1


;
;--------------------------------------------------------------------
;
; Help text

Help:
        MOV   DPTR,#HlpTxt
        LCALL STXT
        LJMP  CMND
;
;--------------------------------------------------------------------
;
; Download
;
ACK  EQU 006H
NACK EQU 015H
;
HEXIN:  MOV   A,#ACK          ; acknowledge request
        LCALL SND
        LCALL GET17           ; get byte count
        MOV   HEXLEN,@R0
        MOV   R0,#HEXSUM      ; checksum:=0
        LCALL CLR16
        LCALL GET17           ; get start address
        MOV   DP0L,@R0
        INC   R0
        MOV   DP0H,@R0
        DEC   R0
hexlp:  LCALL GET17           ; get successive bytes
        MOV   A,@R0
        MOVX  @DPTR,A         ; store at address
        INC   DPTR
        MOV   A,R0
        MOV   R1,A            ; exchange pointer
        MOV   R0,#HEXSUM      ; add value in SUM
        LCALL ADD16
        DJNZ  HEXLEN,hexlp
        LCALL GET17           ; get checksum
        MOV   R1,#HEXSUM
        MOV   A,@R0           ; compare with computed checksum
        XRL   A,@R1
        JNZ   hexerr
        INC   R0
        INC   R1
        MOV   A,@R0
        XRL   A,@R1
        JNZ   hexerr
hexok:  MOV   A,#ACK          ; if ok then acknowledge
        LCALL SND
        RET

hexerr: MOV   A,#NACK         ; else negative acknowledge
        LCALL SND
        RET
;
;--------------------------------------------------------------------
; Default IRQ-Adresse in CodeRAM eintragen
LINK:   MOV   R7,DP0L          ; target adr in DPTR , table offset in A
        MOV   R6,DP0H
        MOV   DPTR,#CodeRAM   ; low byte = 0
        MOV   DP0L,A     
        MOV   A,#2            ; LJMP instruction
        MOVX  @DPTR,A
        INC   DPTR
        MOV   A,R6
        MOVX  @DPTR,A
        INC   DPTR
        MOV   A,R7
        MOVX  @DPTR,A
        RET
;
;------------------------------------------------------------------------
;
; function codes for MON calls
;
ccCHR   EQU  001H
ccSTXT  EQU  002H
ccBYTE  EQU  003H
ccHEX16 EQU  004H
ccdRO16 EQU  005H
ccdRO32 EQU  006H
cchRO16 EQU  007H
cchRO32 EQU  008H
;
ccGETC  EQU  010H
ccTSTC  EQU  011H
ccGET16 EQU  012H
ccGET10 EQU  013H
ccGETU  EQU  014H
;
ccSNAP  EQU  020H
ccLTIME EQU  021H
ccSTIME EQU  022H
;
ccLINK  EQU  040H
;
ccADD   EQU  050H
ccSUB   EQU  051H
ccMUL   EQU  052H
ccDIV   EQU  053H
ccABS   EQU  054H
ccNEG   EQU  055H
;
;------------------------------------------------------------------------
;
; main monitor service routine
;
MON:     MOV   ACCSAV,A                 ; do not change first lines !
         MOV   PSWSAV,PSW
         MOV   A,COMMAND
ttSTIME: CJNE  A,#ccSTIME,ttLTIME
         LCALL STIME
         LJMP  ttRet

ttLTIME: CJNE  A,#ccLTIME,ttSNAP
         LCALL LTIME                    ; gives delay of
         LJMP  ttRet

ttSNAP:  CJNE  A,#ccSNAP,ttSTXT
         LJMP  SNAP

ttSTXT:  CJNE  A,#ccSTXT,ttBYTE
         LJMP  STXT
ttBYTE:  CJNE  A,#ccBYTE,ttCHR
         MOV   A,ACCSAV
         LJMP  BYTE
ttCHR:   CJNE  A,#ccCHR,ttHEX16
         MOV   A,ACCSAV
         LJMP  SND
ttHEX16: CJNE A,#ccHEX16,ttdRO16
         MOV   A,DP0H
         LCALL BYTE
         MOV   A,DP0L
         LJMP  BYTE
ttdRO16: CJNE  A,#ccdRO16,ttdRO32
         LJMP  dDSP16
ttdRO32: CJNE  A,#ccdRO32,tthRO16
         LJMP  dDSP32
tthRO16: CJNE  A,#cchRO16,tthRO32
         LJMP  hDSP16
tthRO32: CJNE  A,#cchRO32,ttGETC
         LJMP  hDSP32
ttGETC:  CJNE  A,#ccGETC,ttTSTC
         LJMP  GETCHR
ttTSTC:  CJNE  A,#ccTSTC,ttGET10
         LJMP  TSTC
ttGET10: CJNE  A,#ccGET10,ttGET16
         LJMP  GET10
ttGET16: CJNE  A,#ccGET16,ttGETU
         LJMP  GET16
ttGETU:  CJNE  A,#ccGETU,ttLINK
         LJMP  GETUPC
ttLINK:  CJNE  A,#ccLINK,ttADD
         LJMP  LINK1
ttADD:   CJNE  A,#ccADD,ttSUB
         LJMP  rADD16
ttSUB:   CJNE  A,#ccSUB,ttMUL
         LJMP  rSUB16
ttMUL:   CJNE  A,#ccMUL,ttDIV
         LJMP  rMUL16
ttDIV:   CJNE  A,#ccDIV,ttABS
         LJMP  rDIV16
ttABS:   CJNE  A,#ccABS,ttNEG
         LJMP  rABS16
ttNEG:   CJNE  A,#ccNEG,ttERR1
         LJMP  rNEG16
ttERR1:  MOV   DPTR,#ERRTXT1
         LCALL STXT
         LJMP 0
ERRTXT1: DB 'illegal call to MON ! ABORT',13,10,0
;
;------------------------------------------------------------------------
;
; Short time delay
;
STIME:   MOV   A,DP0L     ; 6+DPTR*15 cycles (MOV DPTR + CALL adds 4)
         ORL   A,DP0H     ; (6+DPTR) * (1.0, 0.54, 0.5) s at (12, 22.1184, 24 MHz)
         JZ    SRET1
         SETB  C
         MOV   A,DP0L
         SUBB  A,#0
         MOV   DP0L,A
         MOV   A,DP0H
         SUBB  A,#0
         MOV   DP0H,A
         NOP
         NOP
         SJMP  STIME
SRET1:   RET
;
; Long Time delay
;
LTIME:   MOV   A,DP0L     ; 6+DPTR*(15 + TIME0) micros (MOV DPTR + CALL add 4)
         ORL   A,DP0H     ; (6+DPTR) * 500 micros
         JZ    SRET2
         SETB  C
         MOV   A,DP0L
         SUBB  A,#0
         MOV   DP0L,A
         MOV   A,DP0H
         SUBB  A,#0
         MOV   DP0H,A
         ACALL TIME0
         SJMP  LTIME
SRET2:   RET


; Total = 5 * (n1+1) cyles  
;   12 MHz:      NL =  98 ==> 495 s
;   22,1184 MHz: NL = 182 ==> 496 s
;   24 MHz:      NL = 197 ==> 495 s
;                           =   
TIME0:   MOV   DPLSAV,#NL      ; 2 cycles
         NOP                   ; 1
TIME0L:  NOP                   ; 1
         NOP                   ; 1
         NOP                   ; 1
         DJNZ  DPLSAV,TIME0L   ; 2
         RET                   ; 2
;
;------------------------------------------------------------------------
;
LINK1:   MOV   A,ACCSAV       ; restore A
         LJMP LINK            ; use old MON1 routine
;
;------------------------------------------------------------------------
; display snapshot
SNAP:    MOV   DPLSAV,DP0L     ; PSW and ACC have already been saved
         MOV   DPHSAV,DP0H
         MOV   DPTR,#SNAPTXT
         LCALL STXT
;
         MOV   A,ACCSAV       ; display ACCUS
         LCALL BYTE
         LCALL BLANK
;
         MOV   A,B
         LCALL BYTE
         LCALL BLANK
;
         LCALL BLANK          ; PSW comes from PSWSAV
         MOV   A,PSWSAV
         LCALL BYTE
         LCALL BLANK
;
         LCALL BLANK          ; display SP
         MOV   A,SP
         LCALL BYTE
         LCALL BLANK
;
         MOV   A,DPHSAV       ; display DPH,DPL
         LCALL BYTE
         MOV   A,DPLSAV
         LCALL BYTE
;
         LCALL BLANK          ; display R0 .. R7
         MOV   A,R0
         LCALL BYTE
         LCALL BLANK
         MOV   A,R1
         LCALL BYTE
         LCALL BLANK
         MOV   A,R2
         LCALL BYTE
         LCALL BLANK
         MOV   A,R3
         LCALL BYTE
         LCALL BLANK
         MOV   A,R4
         LCALL BYTE
         LCALL BLANK
         MOV   A,R5
         LCALL BYTE
         LCALL BLANK
         MOV   A,R6
         LCALL BYTE
         LCALL BLANK
         MOV   A,R7
         LCALL BYTE
;
         MOV   DP0H,DPHSAV
         MOV   DP0L,DPLSAV
ttRet:   MOV   A,ACCSAV
         MOV   PSW,PSWSAV
         RET

SNAPTXT: DB    13,10,' A  B PSW  SP DPTR R0 R1 R2 R3 R4 R5 R6 R7'
         DB    13,10,0
;
;------------------------------------------------------------------------
;
; Arithmetic routines
;
rADD16: MOV   A,@R0              ; @R0 + @R1 goes to top
        ADD   A,@R1
        MOV   top,A
        INC   R0
        INC   R1
        MOV   A,@R0
        ADDC  A,@R1
        MOV   top+1,A
        MOV   R0,#top
        RET
;
rSUB16: CLR   C                 ; @R0 - @R1 goes to top
        MOV   A,@R0
        SUBB  A,@R1
        MOV   top,A
        INC   R0
        INC   R1
        MOV   A,@R0
        SUBB  A,@R1
        MOV   top+1,A
        MOV   R0,#top
        RET
;
rMUL16: MOV   FAC1+0,@R0        ; @R0 * @R1 goes to PROD
        MOV   FAC2+0,@R1
        INC   R0
        INC   R1
        MOV   FAC1+1,@R0
        MOV   FAC2+1,@R1
        CALL  MUL16
        MOV   R0,#PROD       ; result pointer in R0
        RET
;
rDIV16: MOV   PROD+0,@R0
        INC   R0
        MOV   PROD+1,@R0
        INC   R0
        MOV   PROD+2,@R0
        INC   R0
        MOV   PROD+3,@R0
        MOV   DIVI,@R1
        INC   R1
        MOV   DIVI+1,@R1
        ACALL DIV16
        MOV   R0,#QUOT       ; quotient pointer
        MOV   R1,#PROD+2     ; remainder pointer
        RET
;
rABS16: INC  R0              ; is ABS set CY if negative
        MOV  A,@R0
        JNB  ACC.7,noneg1
        DEC  R0
        SJMP rNEG16
noneg1: CLR  C
        RET
;
rNEG16: MOV  A,@R0           ; set negative @ R0
        XRL  A,#0FFH
        ADD  A,#1
        MOV  @R0,A
        INC  R0
        MOV  A,@R0
        XRL  A,#0FFH
        ADDC A,#0
        MOV  @R0,A
        SETB C
        RET
;
;--------------------------------------------------------------------
;
; 32 / 16 bit division routine
;
DIV16:  MOV   PROD+4,#00H     ; clear extension  approx. 1000 microsec
        MOV   DIVCNT,#16      ; 15 normal steps, last step: remainder !
DIVLP:  ACALL SHR5
        ACALL TRYSUB          ; try and perform subtraction, results in CY
        ACALL SHLQ            ; shift CY into QUOTIENT
        DJNZ  DIVCNT,DIVLP
        RET
;
TRYSUB: CLR  C           ; no borrow
        MOV  A,PROD+2
        SUBB A,DIVI+0
        MOV  A,PROD+3
        SUBB A,DIVI+1
        MOV  A,PROD+4    ; is accu extension
        SUBB A,#0
        JNC  SUBIT
        RET

SUBIT:  MOV  A,PROD+2    ; now really subtract  , we have no borrow from before
        SUBB A,DIVI+0
        MOV  PROD+2,A
        MOV  A,PROD+3
        SUBB A,DIVI+1
        MOV  PROD+3,A
        MOV  A,PROD+4    ; is accu extension
        SUBB A,#0
        MOV  PROD+4,A
        RET
;
SHLQ:   CPL C
        MOV  A,QUOT+0    ; use carry from before and shift into quotient
        RLC  A
        MOV  QUOT+0,A
        MOV  A,QUOT+1
        RLC  A
        MOV  QUOT+1,A
        RET
 ;
SHR5:   CLR  C           ; shift 5 bytes = product+ACC 17 bit extension
        MOV  A,PROD+0
        RLC  A
        MOV  PROD+0,A
        MOV  A,PROD+1
        RLC  A
        MOV  PROD+1,A
        MOV  A,PROD+2
        RLC  A
        MOV  PROD+2,A
        MOV  A,PROD+3
        RLC  A
        MOV  PROD+3,A
        MOV  A,PROD+4
        RLC  A
        MOV  PROD+4,A
        RET
;
;------------------------------------------------------------------------
;
MUL16:  MOV  A,FAC1+0    ; 16*16 multply -> 32 bit  ( approx. 70 microsec )
        MOV  B,FAC2+0
        MUL  AB          ; low * low byte
        MOV  PROD+0,A
        MOV  PROD+1,B
        MOV  A,FAC1+1
        MOV  B,FAC2+1
        MUL  AB          ; high * high byte
        MOV  PROD+2,A
        MOV  PROD+3,B
        MOV  A,FAC1+1    ; mixed prod 1
        MOV  B,FAC2+0
        MUL  AB
        ADD  A,PROD+1    ; add 3 byte wide
        MOV  PROD+1,A
        MOV  A,PROD+2
        ADDC A,B
        MOV  PROD+2,A
        MOV  A,PROD+3
        ADDC A,#0
        MOV  PROD+3,A
        MOV  A,FAC1+0    ; mixed prod 2
        MOV  B,FAC2+1
        MUL  AB
        ADD  A,PROD+1    ; add 3 byte wide
        MOV  PROD+1,A
        MOV  A,PROD+2
        ADDC A,B
        MOV  PROD+2,A
        MOV  A,PROD+3
        ADDC A,#0
        MOV  PROD+3,A
        RET
;
;------------------------------------------------------------------------
; long 32 bit dec display of value at R0
dDSP32: LCALL dCNVa      ; prepare conversion buffer
        LCALL dCNVb      ; convert 4 bytes
        LCALL dCNVb
        LCALL dCNVb
        LCALL dCNVb
        MOV   R1,#dACCU+dBYTS
        MOV   R6,#dBYTS
        LCALL DDSTR      ; display string
        RET

; long 32 bit hex display of value at R0
hDSP32: INC   R0
        INC   R0
        INC   R0
        MOV   A,@R0      ; point to most significant byte
        LCALL BYTE
        DEC   R0
        MOV   A,@R0
        LCALL BYTE
        DEC   R0
        MOV   A,@R0
        LCALL BYTE
        DEC   R0
        MOV   A,@R0
        LCALL BYTE
        RET

; 16 bit display hexadecimal
hDSP16: INC   R0           ; point to higher byte
        MOV   A,@R0
        LCALL BYTE       ; display
        DEC   R0
        MOV   A,@R0
        LCALL BYTE       ; display lower byte
        RET

; word 16 bit dec display of value at R0
dDSP16: LCALL dCNVa      ; prepare conversion
        LCALL dCNVb      ; convert two bytes
        LCALL dCNVb
        MOV   R1,#dACCU+3
        MOV   R6,#3
        CALL  DDSTR      ; display 3 BCD bytes so 65535 fits
        RET
;
;--------------------------------------------------------------------
;
; hexa to decimal conversion
;
dCNVa:  MOV   A,#0
        MOV   dACCU+0,A  ; init dec conversion dACCU:=0 ; dREG1:=1
        MOV   dACCU+1,A
        MOV   dACCU+2,A
        MOV   dACCU+3,A
        MOV   dACCU+4,A
        MOV   dREG1+0,#1
        MOV   dREG1+1,A
        MOV   dREG1+2,A
        MOV   dREG1+3,A
        MOV   dREG1+4,A
        RET
;
dCNVb:  MOV   A,@R0      ; convert Byte at R0 , inc R0
        INC   R0
DCNVb1: MOV   R2,A       ; convert Byte ACC
        MOV   R6,#8      ; convert R2 using dACCU and dREG1
dCNV1:  MOV   A,R2       ; get bit from R2
        RRC   A
        MOV   R2,A
        JNC   dCNV2
        LCALL ADD1       ; if CY add dREG1 to dACCU using BCD
dCNV2:  LCALL DOUBL1     ; dREG1:=2*dREG1
        DJNZ  R6,dCNV1   ; loop until all 8 bits done
        RET
;
DOUBL1: MOV   A,dREG1+0  ; dREG1:=2*dREG1 , BCD coded 5 bytes
        ADD   A,dREG1+0
        DA    A
        MOV   dREG1+0,A
        MOV   A,dREG1+1
        ADDC  A,dREG1+1
        DA    A
        MOV   dREG1+1,A
        MOV   A,dREG1+2
        ADDC  A,dREG1+2
        DA    A
        MOV   dREG1+2,A
        MOV   A,dREG1+3
        ADDC  A,dREG1+3
        DA    A
        MOV   dREG1+3,A
        MOV   A,dREG1+4
        ADDC  A,dREG1+4
        DA    A
        MOV   dREG1+4,A
        RET
;
ADD1:   MOV   A,dACCU+0  ; dACCU:=dACCU+dREG1 , bcd coded 5 bytes
        ADD   A,dREG1+0
        DA    A
        MOV   dACCU+0,A
        MOV   A,dACCU+1
        ADDC  A,dREG1+1
        DA    A
        MOV   dACCU+1,A
        MOV   A,dACCU+2
        ADDC  A,dREG1+2
        DA    A
        MOV   dACCU+2,A
        MOV   A,dACCU+3
        ADDC  A,dREG1+3
        DA    A
        MOV   dACCU+3,A
        MOV   A,dACCU+4
        ADDC  A,dREG1+4
        DA    A
        MOV   dACCU+4,A
        RET
;
DDSTR:  NOP              ; display R6 bytes in decreasing order @R1
dDSPlp: DEC   R1         ; used for decimal or hexadecimal output
        MOV   A,@R1
        LCALL BYTE
        DJNZ  R6,dDSPlp
        RET
;
;------------------------------------------------------------------------
; get a Byte withou ECHO
GET8:   LCALL GETHX1    ; 1st hex value
        JC    err8
        RL    A
        RL    A
        RL    A
        RL    A         ; *16
        ANL   A,#0F0H
        MOV   R7,A
        LCALL GETHX1    ; 2nd hex value
        JC    err8
        ORL   A,R7      ; add
        CLR   C
err8:   RET

;
GET17:  MOV   R0,#dACCU  ; get hex number without echo
        LCALL CLR16      ; compare with GET16
get17l: LCALL GETHX1
        JNC   OK17
        MOV   R0,#dACCU
        RET
OK17:   MOV   R7,A
        LCALL SHL16      ; *16
        LCALL SHL16
        LCALL SHL16
        LCALL SHL16
        MOV   A,R7
        ADD   A,@R0
        MOV   @R0,A
        SJMP  get17l

; get char. without echo, convert from hex
GETHX1: LCALL GETUPC
        SJMP  GOON2

; char with echo, convert from hex
GETHEX: LCALL GETUPC
        LCALL ECHO
GOON2:  CLR   C
        SUBB  A,#'0'
        JC    hcor1    ; < '0'
        SUBB  A,#10
        JNC   nohex1   ; > 9
        ADD   A,#10    ; corr.
ISHEX:  CLR   C
        RET

nohex1: SUBB  A,#7     ; A,..,F
        JC    hcor2
        SUBB  A,#6
        JNC   hcor3
        ADD   A,#16
        SJMP  ISHEX    ; o.k.
nohex:  SETB   C
        RET
        
hcor1:  ADD    A,#'0'
        SJMP nohex
hcor2:  ADD    A,#'0'+17
        SJMP nohex
hcor3:  ADD    A,#'0'+17+7
        SJMP nohex
;
ECHO:   CJNE  A,#13,EC1
        LCALL SND
        MOV A,#10
EC1:    LJMP SND
;
GET16:  MOV   R0,#dACCU  ; get hex value int dACCU , count digits in R6
        LCALL CLR16      ; value:=0
        MOV   A,#'H'     ; send prompt
        LCALL SND
        MOV   A,#'>'
        LCALL SND
        MOV   R6,#0      ; count significant hex digits
get16l: LCALL GETHEX     ; get input character
        JNC   OK1        ; go on if hexadecimal
        MOV   R0,#dACCU  ; return address of accu also
        RET

OK1:    INC   R6         ; update number of digits
        MOV   R7,A       ; save input value
        LCALL SHL16      ; dACCU:=dACCU * 16
        LCALL SHL16
        LCALL SHL16
        LCALL SHL16
        MOV   A,R7
        ADD   A,@R0      ; add input value ( no CY )
        MOV   @R0,A
        SJMP  get16l
;
;------------------------------------------------------------------------
;
GET10:  MOV   R0,#dACCU  ; get decimal value int ACCU0 , digits in R6
        LCALL CLR16      ; value:=0
        MOV   A,#'D'     ; prompt
        LCALL SND
        MOV   A,#'>'
        LCALL SND
        MOV   R6,#0      ; significant hex digits
get10l: LCALL GETDEC
        JNC   OK1d       ; go on if ok
        MOV   R0,#dACCU  ; return address of accu also
        RET

OK1d:   INC   R6         ; update digit count
        MOV   R7,A       ; saves chr value
        MOV   R2,dACCU   ; LSB
        MOV   R3,dACCU+1 ; MSB
        MOV   A,R2       ; multiply by 10 this value
        MOV   B,#10
        MUL   AB
        MOV   dACCU,A
        MOV   dACCU+1,B
        MOV   A,R3
        MOV   B,#10
        MUL   AB
        ADD   A,dACCU+1  ; add to old MSB
        MOV   dACCU+1,A
        MOV   A,R7       ; char value
        ADD   A,dACCU    ; add input value
        MOV   dACCU,A
        MOV   A,dACCU+1
        ADDC  A,#0       ; account for CY
        MOV   dACCU+1,A
        SJMP  get10l
;
GETDEC: LCALL GETUPC     ; get decimal character, set CY if not
        LCALL ECHO
        CLR   C
        SUBB  A,#'0'
        JC    nodec
        SUBB  A,#10
        JNC   nodec
        ADD   A,#10
        CLR   C
        RET
nodec:  SETB   C
        RET
;
;------------------------------------------------------------------------
;
; 16 bit arithmetic for internal monitor use
; 16 bit value stored in order low,high byte
;
CLR16:  MOV   @R0,#0
        INC   R0
        MOV   @R0,#0
        DEC   R0
        RET
;
ADD16:  MOV   A,@R0      ; start at low digits
        ADD   A,@R1
        MOV   @R0,A
        INC   R0         ; advance pointers
        INC   R1
        MOV   A,@R0
        ADDC  A,@R1      ; with CY from last ADD
        MOV   @R0,A
        DEC   R0         ; restore pointers
        DEC   R1
        RET
;
SHL16:  MOV   A,@R0      ; shift left 16 bit value
        ADD   A,@R0
        MOV   @R0,A
        INC   R0
        MOV   A,@R0
        ADDC  A,@R0
        MOV   @R0,A
        DEC   R0
        RET
;
;------------------------------------------------------------------------
; low level IO routines

; send text from program store at DPTR
STXT:   MOV   A,#0       ; destroy ACC and DPTR
        MOVC  A,@A+DPTR
        JZ    ETXT
        LCALL SND
        INC   DPTR
        SJMP  STXT
ETXT:   RET
;
BLANK:  MOV   A,#' '     ; send BLANK
        SJMP  SND
;
BYTE:   PUSH  ACC        ; send BYTE hexadecimal , destroy ACC only
        SWAP  A
        LCALL NIBBLE
        POP   ACC
NIBBLE: ANL   A,#0FH
        ADD   A,#246
        JC    HEXOUT
        ADD   A,#58
        SJMP  SND
HEXOUT: ADD   A,#65
        SJMP  SND
;
CRLF:   MOV   A,#13      ; send a CRLF
        LCALL SND
        MOV   A,#10
        LCALL SND
        RET
;
SND:    JNB   SCON.1,SND ; must not destroy any reg
        CLR   SCON.1
        MOV   SBUF,A
        CJNE  A,#10,OK2
WAITCR: MOV   CNT1,#100  ; wait for slow scrolling terminals
LOP1:   MOV   A,#255
LOP2:   DJNZ  ACC,LOP2
        DJNZ  CNT1,LOP1
        MOV   A,#10
OK2:    RET
;
GETUPC: LCALL GETCHR     ; get upper case character from serial port
        LCALL UPCASE
        RET
;
UPCASE: CLR   C          ; convert character to upper case letter
        SUBB  A,#'a'
        JC    UPC1       ; char <97  a-> 0    z->25
        SUBB  A,#26
        JC    UPC2       ; (char-a)<26  , do upcase translation
        ADD   A,#26+'a'
        RET
UPC2:   ADD   A,#26+'A'
        RET
UPC1:   ADD   A,#'a'
        RET

; get character from serial port
GETCHR: JNB   SCON.0,GETCHR
        CLR   SCON.0
        MOV   A,SBUF
        RET
;
TSTC:   JB    SCON.0,isther   ; test if character there return 1 else 0
        MOV   A,#0
        RET
isther: MOV   A,#1
        RET


;********************************************************************************************************
; Extracted from PaulMon

;to do automatic baud rate detection, we assume the user will
;press the carriage return, which will cause this bit pattern
;to appear on port 3 pin 0 (CR = ascii code 13, assume 8N1 format)
;
;              0 1 0 1 1 0 0 0 0 1
;              | |             | |
; start bit----+ +--lsb   msb--+ +----stop bit
;
;we'll start timer #1 in 16 bit mode at the transition between the
;start bit and the LSB and stop it between the MBS and stop bit.
;That will give approx the number of cpu cycles for 8 bits.  Divide
;by 8 for one bit and by 16 since the built-in UART takes 16 timer
;overflows for each bit.  We need to be careful about roundoff during
;division and the result has to be inverted since timer #1 counts up.  Of
;course, timer #1 gets used in 8-bit auto reload mode for generating the
;built-in UART's baud rate once we know what the reload value should be.


AutoBaud:
        mov     a, #BaudCount  ;skip if user supplied baud rate constant
        jnz     autoend_jmp
        mov     a, BaudSave+3  ;is there a value from a previous boot?
        xrl     BaudSave+2, #01010101b
        xrl     BaudSave+1, #11001100b
        xrl     BaudSave+0, #00011101b
        cjne    a, BaudSave+2, autob1
        cjne    a, BaudSave+1, autob1
        cjne    a, BaudSave+0, autob1
autoend_jmp:
        ajmp    autoend

autob1: ;wait for inactivity

        mov     pcon, #80H     ;configure uart, fast baud
        mov     scon, #42H     ;configure uart, but receive disabled
        mov     tmod, #11H     ;get timers ready for action (16 bit mode)
        clr     a
        mov     tcon, a
        mov     tl0, a
        mov     th0, a
        mov     tl1, a
        mov     th1, a

        ;make sure there is no activity on the line
        ;before we actually begin looking for the carriage return
        mov     r0, #200
autob1b:mov     r1, #30
autob1c:jnb     p3.0, autob1
        djnz    r1, autob1c
        djnz    r0, autob1b

autob2: ;look for the bits of the carriage return
        jb      p3.0, autob2    ;wait for start bit
        jb      p3.0, autob2
        jb      p3.0, autob2    ;  check it a few more times to make
        jb      p3.0, autob2    ;  sure we don't trigger on some noise
        jb      p3.0, autob2
autob2b:jnb     p3.0, autob2b   ;wait for bit #0 to begin
        setb    tr1             ;and now we're timing it
autob2c:jb      tf1, autob1     ;check for timeout while waiting
        jb      p3.0, autob2c   ;wait for bit #1 to begin
autob2d:jb      tf1, autob1     ;check for timeout while waiting
        jnb     p3.0, autob2d   ;wait for bit #2 to begin
autob2e:jb      tf1, autob1     ;check for timeout while waiting
        jb      p3.0, autob2e   ;wait for bit #4 to begin
        setb    tr0             ;start timing last 4 bits
autob2f:jb      tf1, autob1     ;check for timeout while waiting
        jnb     p3.0, autob2f   ;wait for stop bit to begin
        clr     tr1             ;stop timing (both timers)
        clr     tr0

        jb      tf1, autob1     ;check for timeout one last time

        ;compute the baud rate based on timer1
        mov     a, tl1
        rlc     a
        mov     b, a
        mov     a, th1
        rlc     a
        jc      autob1          ;error if timer0 > 32767
        mov     c, b.7
        addc    a, #0
        cpl     a
        inc     a               ;now a has the value to load into th1
        jz      autob1          ;error if baud rate too fast

        ;after we get the carriage return, we need to make sure there
        ;isn't any "crap" on the serial line, as there is in the case
        ;were we get the letter E (and conclude the wrong baud rate).
        ;unfortunately the simple approach of just looking at the line
        ;for silence doesn't work, because we have to accept the case
        ;where the user's terminal emulation is configured to send a
        ;line feed after the carriage return.  The best thing to do is
        ;use the uart and look see if it receives anything

autob3: mov     th1, a          ;config timer1
        mov     tl1, #255       ;start asap!
        mov     tmod, #21H     ;autoreload mode
        setb    ren             ;turn on the uart
        setb    tr1             ;turn on timer1 for its clock

        mov     a, th1
        cpl     a
        inc     a
        mov     r1, a
autob3b:mov     r0, #255
autob3c:djnz    r0, autob3c
        djnz    r1, autob3b

        jnb     ri, autob4
        ;if we got here, there was some stuff after the carriage
        ;return, so we'll read it and see if it was the line feed
        clr     ri
        mov     a, sbuf
        anl     a, #01111111b
        add     a, #246
        jz      autob4          ;ok if 0A, the line feed character
        add     a, #5
        jz      autob4          ;of if 05, since we may have missed start bit
autob1_jmp:
        ljmp    autob1
autob4:
        ;compute the baud rate based on timer0, check against timer1 value
        mov     a, tl0
        rlc     a
        mov     r0, a
        mov     a, th0
        rlc     a
        mov     r1, a
        jc      autob1_jmp      ;error if timer0 > 32767
        mov     a, r0
        rlc     a
        mov     b, a
        mov     a, r1
        rlc     a
        mov     c, b.7
        addc    a, #0
        jz      autob1_jmp      ;error if baud too fast!
        cpl     a
        inc     a
        cjne    a, th1, autob1_jmp
        ;acc has th1 value at this point

autoend:mov     BaudSave+3, a
        mov     BaudSave+2, a  ;store the baud rate for next warm boot.
        mov     BaudSave+1, a
        mov     BaudSave+0, a
        xrl     BaudSave+2, #01010101b
        xrl     BaudSave+1, #11001100b
        xrl     BaudSave+0, #00011101b
        ret
;
;----------------------------------------------------------------------
;
        END
